@@ -1,15 +1,17 @@
 #!/usr/bin/perl
-#VERSION,2.01
+use strict;
+
+#VERSION,2.02
 use Getopt::Long;
 Getopt::Long::Configure('no_ignore_case');
 
 ###############################################################################
 #                               Nikto                                         #
 # --------------------------------------------------------------------------- #
-#                       last update: 11.13.2007                               #
+#                       last update: 01.09.2008                               #
 # --------------------------------------------------------------------------- #
 ###############################################################################
-#  Copyright (C) 2004-2007 CIRT, Inc.
+#  Copyright (C) 2004-2008 CIRT, Inc.
 #
 #  This program is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU General Public License
@@ -36,18 +38,19 @@ Getopt::Long::Configure('no_ignore_case');
 # can not be held liable for anything done with this program, code, or items discovered
 # with this program's use.
 #######################################################################
+
 # global var/definitions
-my $STARTTIME = localtime();
 use vars qw/$TEMPLATES %ERRSTRINGS %VERSIONS %CLI %VARIABLES %TESTS $CONTENT %FILES $CURRENT_HOST_ID $CURRENT_PORT/;
-use vars qw/%REALMS %REALMS_TESTED %NIKTOCONFIG %NIKTO %OUTPUT %SERVER %request %result %COUNTERS/;
-use vars qw/@KB_MSGS %db_extensions %FoF %UPDATES $DIV %TARGETS @DBFILE @SERVERFILE @BUILDITEMS $PROXYCHECKED/;
+use vars qw/%REALMS %REALMS_TESTED %NIKTOCONFIG %NIKTO %OUTPUT %SERVER %request %result %COUNTERS $STARTTIME/;
+use vars qw/%db_extensions %FoF %UPDATES $DIV %TARGETS @DBFILE @SERVERFILE @BUILDITEMS $PROXYCHECKED $http_eol/;
 
 # setup
+$STARTTIME         = localtime();
 $DIV               = "-" x 75;
-$NIKTO{version}    = "2.01";
+$NIKTO{version}    = "2.02";
 $NIKTO{name}       = "Nikto";
 $NIKTO{configfile} = "config.txt";    ### Change this line if your setup is having trouble finding it
-$http_eol="\r\n";
+$http_eol          = "\r\n";
 
 # read the --config option
 {
@@ -60,15 +63,16 @@ $http_eol="\r\n";
 
 load_configs();
 find_plugins();
-require "$NIKTO{plugindir}/nikto_core.plugin";       ### Change this line if your setup is having trouble finding it
-nprint("T:$STARTTIME: Starting","d");
+require "$NIKTO{plugindir}/nikto_core.plugin";    ### Change this line if your setup is having trouble finding it
+nprint("T:$STARTTIME: Starting", "d");
 require "$NIKTO{plugindir}/nikto_reports.plugin";    ### Change this line if your setup is having trouble finding it
 require "$NIKTO{plugindir}/nikto_single.plugin";     ### Change this line if your setup is having trouble finding it
 require "$NIKTO{plugindir}/LW2.pm";                  ### Change this line if your setup is having trouble finding it
+
 # use LW2;					     ### Change this line to use a different installed version
 
-($a,$b) = split(/\./, $LW2::VERSION);
-die("- You must use LW2 2.4 or later\n") if($a != 2 || $b < 4);
+($a, $b) = split(/\./, $LW2::VERSION);
+die("- You must use LW2 2.4 or later\n") if ($a != 2 || $b < 4);
 
 general_config();
 load_databases();
@@ -79,7 +83,7 @@ $request{'whisker'}->{'ssl_save_info'}              = 1;
 $request{'whisker'}->{'lowercase_incoming_headers'} = 1;
 $request{'whisker'}->{'timeout'}                    = $CLI{timeout} || 10;
 if ($CLI{evasion} ne "") { $request{'whisker'}->{'encode_anti_ids'} = $CLI{evasion}; }
-$request{'User-Agent'}         = $NIKTO{useragent};
+$request{'User-Agent'} = $NIKTO{useragent};
 $request{'whisker'}->{'retry'} = 0;
 proxy_setup();
 
@@ -96,27 +100,38 @@ foreach $CURRENT_HOST_ID (sort { $a <=> $b } keys %TARGETS)
 {
     LW2::http_reset();
     $COUNTERS{hosts_completed}++;
-    @KB_MSGS = ();
     if (($CLI{findonly}) && ($COUNTERS{hosts_completed} % 10) eq 0) { nprint("($COUNTERS{hosts_completed} of $COUNTERS{hosts_total})"); }
 
-    ($TARGETS{$CURRENT_HOST_ID}{hostname}, $TARGETS{$CURRENT_HOST_ID}{ip}, $TARGETS{$CURRENT_HOST_ID}{display_name}) = resolve($TARGETS{$CURRENT_HOST_ID}{ident});
+    ($TARGETS{$CURRENT_HOST_ID}{hostname}, $TARGETS{$CURRENT_HOST_ID}{ip}, $TARGETS{$CURRENT_HOST_ID}{display_name}) =
+      resolve($TARGETS{$CURRENT_HOST_ID}{ident});
     if ($TARGETS{$CURRENT_HOST_ID}{ident} eq "") { next; }
     port_scan($TARGETS{$CURRENT_HOST_ID}{ports_in});
 
     # make sure we have open ports on this target
-    if (keys(%{$TARGETS{$CURRENT_HOST_ID}{ports}}) eq 0)
-   	{ nprint("+ No HTTP(s) ports found on $TARGETS{$CURRENT_HOST_ID}{ident} / $TARGETS{$CURRENT_HOST_ID}{ports_in}","","kb"); }
+    if (keys(%{ $TARGETS{$CURRENT_HOST_ID}{ports} }) eq 0)
+    {
+        nprint("+ No HTTP(s) ports found on $TARGETS{$CURRENT_HOST_ID}{ident}");
+        next;
+    }
 
     $request{'whisker'}->{'host'} = $TARGETS{$CURRENT_HOST_ID}{hostname} || $TARGETS{$CURRENT_HOST_ID}{ip};
-    if ($TARGETS{$CURRENT_HOST_ID}{vhost} ne '') { $request{'Host'}=$TARGETS{$CURRENT_HOST_ID}{vhost}; }
+    if ($TARGETS{$CURRENT_HOST_ID}{vhost} ne '') { $request{'Host'} = $TARGETS{$CURRENT_HOST_ID}{vhost}; }
 
-    foreach $CURRENT_PORT (keys %{ $TARGETS{$CURRENT_HOST_ID}{ports} })
+    foreach $CURRENT_PORT (split(',', $TARGETS{$CURRENT_HOST_ID}{ports_in}))
     {
         if ($CURRENT_PORT eq "") { next; }
+        elsif (!defined($TARGETS{$CURRENT_HOST_ID}{ports}{$CURRENT_PORT}))
+        {
+            nprint("+ No HTTP(s) server found on $TARGETS{$CURRENT_HOST_ID}{ident} / $CURRENT_PORT");
+            next;
+        }
         $request{'whisker'}->{'port'}    = $CURRENT_PORT;
         $request{'whisker'}->{'ssl'}     = $TARGETS{$CURRENT_HOST_ID}{ports}{$CURRENT_PORT}{ssl};
         $request{'whisker'}->{'version'} = $NIKTOCONFIG{DEFAULTHTTPVER};
         if ($NIKTOCONFIG{'STATIC-COOKIE'} ne "") { $request{'Cookie'} = $NIKTOCONFIG{'STATIC-COOKIE'}; }
+        $TARGETS{$CURRENT_HOST_ID}{total_vulns} = 0;
+        delete $TARGETS{$CURRENT_HOST_ID}{positives};
+        %FoF = ();
 
         get_banner();
 
@@ -129,21 +144,16 @@ foreach $CURRENT_HOST_ID (sort { $a <=> $b } keys %TARGETS)
             }
             if ($TARGETS{$CURRENT_HOST_ID}{ports}{$CURRENT_PORT}{ssl}) { $protocol .= "s"; }
             nprint("+ Server: $protocol://$TARGETS{$CURRENT_HOST_ID}{display_name}:$CURRENT_PORT\t$TARGETS{$CURRENT_HOST_ID}{ports}{$CURRENT_PORT}{banner}");
-        }
-	else 
-	{
+        } else
+        {
             dump_target_info();
-            %FoF = ();
-            $TARGETS{$CURRENT_HOST_ID}{total_vulns} = 0;
             auth_check();
             check_cgi();
             set_scan_items();
             map_codes();
             run_plugins();
             test_target();
-	}
-
-        write_kbase(next_kbase_id());
+        }
         write_output();
     }
 }
@@ -151,7 +161,7 @@ foreach $CURRENT_HOST_ID (sort { $a <=> $b } keys %TARGETS)
 nprint("+ $COUNTERS{hosts_total} host(s) tested");
 send_updates();
 close_output();
-nprint("T:" . localtime() . ": Ending","d");
+nprint("T:" . localtime() . ": Ending", "d");
 exit;
 
 #################################################################################
@@ -187,54 +197,53 @@ sub load_configs
 # find plugins directory
 sub find_plugins
 {
+
     # get the correct path to 'plugins'
     # if defined in config.txt file... most accurate, we hope
     if (($NIKTOCONFIG{EXECDIR} ne "") && (-d "$NIKTOCONFIG{EXECDIR}/plugins"))
     {
-       	$NIKTO{execdir}     = $NIKTOCONFIG{EXECDIR};
-	$NIKTO{plugindir}   = "$NIKTO{execdir}/plugins"; 
-	$NIKTO{templatedir} = "$NIKTO{execdir}/templates";
+        $NIKTO{execdir}     = $NIKTOCONFIG{EXECDIR};
+        $NIKTO{plugindir}   = "$NIKTO{execdir}/plugins";
+        $NIKTO{templatedir} = "$NIKTO{execdir}/templates";
     }
 
     if ($NIKTO{execdir} eq "")
-    {   # try pwd
+    {    # try pwd
         if (-d "$ENV{PWD}/plugins")
         {
-            	$NIKTO{execdir}     = $ENV{PWD};
-        	$NIKTO{plugindir}   = "$NIKTO{execdir}/plugins";
-        	$NIKTO{templatedir} = "$NIKTO{execdir}/templates";
-        } 
-	else {  # try $0
-            	my $EXECDIR = $0;
-            	chomp($EXECDIR);
-            	$EXECDIR =~ s/\/nikto.pl$//;
+            $NIKTO{execdir}     = $ENV{PWD};
+            $NIKTO{plugindir}   = "$NIKTO{execdir}/plugins";
+            $NIKTO{templatedir} = "$NIKTO{execdir}/templates";
+        } else
+        {    # try $0
+            my $EXECDIR = $0;
+            chomp($EXECDIR);
+            $EXECDIR =~ s/\/nikto.pl$//;
 
-        	if (-d "$EXECDIR/plugins")
-        	{
-               	 	$NIKTO{execdir}     = $EXECDIR;
-                	$NIKTO{plugindir}   = "$NIKTO{execdir}/plugins";
-                	$NIKTO{templatedir} = "$NIKTO{execdir}/templates";
-        	} 
+            if (-d "$EXECDIR/plugins")
+            {
+                $NIKTO{execdir}     = $EXECDIR;
+                $NIKTO{plugindir}   = "$NIKTO{execdir}/plugins";
+                $NIKTO{templatedir} = "$NIKTO{execdir}/templates";
+            }
         }
 
-	if ($NIKTO{execdir} eq "")  
-		{ 	# try ./
-			$NIKTO{execdir}= "./"; 
-                	$NIKTO{plugindir}   = "$NIKTO{execdir}/plugins";
-                	$NIKTO{templatedir} = "$NIKTO{execdir}/templates";
-		}
+        if ($NIKTO{execdir} eq "")
+        {    # try ./
+            $NIKTO{execdir}     = "./";
+            $NIKTO{plugindir}   = "$NIKTO{execdir}/plugins";
+            $NIKTO{templatedir} = "$NIKTO{execdir}/templates";
+        }
     }
 
     if (!(-d $NIKTO{plugindir}))
     {
         print STDERR "I can't find 'plugins' directory. I looked around:\n";
-        print STDERR "\t$CONFIG{PLUGINDIR}\n\t$ENV{PWD}\n\t$0\n";
+        print STDERR "\t$NIKTOCONFIG{EXECDIR}\n\t$ENV{PWD}\n\t$0\n";
         print STDERR "Try: switch to the 'nikto' base dir, or\n";
-        print STDERR "Try: set PLUGINDIR in config.txt\n";
+        print STDERR "Try: set EXECDIR in config.txt\n";
         exit;
     }
-
-    $NIKTO{kbase} = "$NIKTO{execdir}/kbase/nikto.kbase";    ### Change this line if you would like to use another KB
     return;
 }
 #################################################################################
